// comprpm -- By Warren E. Downs
// August 30, 1999
// This is a collection of functions for comparing rpm filenames, by version, etc.

// Needed for snprintf()
#define _GNU_SOURCE

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comprpm.h"

#define  min(a,b)	(((a)<(b))?(a):(b))
#define  max(a,b)	(((a)>(b))?(a):(b))

// Returns true if the basenames of the two files are different.
bool different(char *fil1, char *fil2) {
	char			buf1[256];
	char			buf2[256];
	base_name(fil1,buf1,sizeof(buf1));
	base_name(fil2,buf2,sizeof(buf2));
	return strcmp(buf1,buf2);
	}

// Returns true if the version of the first file is newer than that of the second.
bool newer(char *fil1, char *fil2) {
	char			buf1[256];
	char			buf2[256];
	version(fil1,buf1,sizeof(buf1));
	version(fil2,buf2,sizeof(buf2));
	return strcmp(buf1,buf2)>0;
	}

// Returns true if the first base_name is "less than" the second base_name (of the two files).
bool lesser(char *fil1, char *fil2) {
	char			buf1[256];
	char			buf2[256];
	base_name(fil1,buf1,sizeof(buf1));
	base_name(fil2,buf2,sizeof(buf2));
	return strcmp(buf1,buf2)<0;
	}

// Returns true if the first base_name is "greater than" the second base_name (of the two files).
bool greater(char *fil1, char *fil2) {
	char			buf1[256];
	char			buf2[256];
	base_name(fil1,buf1,sizeof(buf1));
	base_name(fil2,buf2,sizeof(buf2));
	return strcmp(buf1,buf2)>0;
	}

// Returns the base_name of a given file, in the provided buffer
void base_name(char *fil, char *base, int size) {
	char 			*pp=fil;
	if(size>0) { *base=0; }
	else       { return; }										// Out of space
	while(*pp) {
		if((*pp=='-') && isdigit(*(pp+1))) {
			int len=min(pp-fil,size-1);
			strncpy(base,fil,len); base[len]=0;
			return;
			}
		pp++;
		}
	strncpy(base,fil,size-1); base[size-1]=0;
	pp=strrchr(base,'.'); if(pp) { *pp=0; }	// Chop off .rpm
	pp=strrchr(base,'.'); if(pp) { *pp=0; }	// Chop off .<architecture>
	}
	
// Returns the version of a given file, in the provided buffer
// Version segments are converted to zero-padded fields, with a width of 4,
// to ensure proper comparison.  For example:
// 		SysVinit-2.76-3mdk.i586.rpm
// would be converted to:
//		000200760003mdk
// (Note the inclusion of mdk-- this means that Mandrake packages (including mdk) will
// be favored over equivalent versions of RedHat (simply due to the longer version string)).
void version(char *fil, char *ver, int size) {
	char 			*pp=fil;
	bool			versioning=false;
	bool			dashed=false;
	if(size>0) { *ver=0; }
	else       { return; }										// Out of space
	
	while(*pp) {
		if(!versioning && (*pp=='-') && isdigit(*(pp+1))) {
			versioning=true; pp++;
			}
		if(versioning && isdigit(*pp)) {
			int val=atoi(pp); while(isdigit(*pp)) { pp++; }		// Skip digits
			char buf[256];
			snprintf(buf,sizeof(buf),"%04d",val);
			if(strlen(ver)+4 < size) { strcat(ver,buf); }
			else					 { break;           }		// Out of space
			if((*pp=='.') || (*pp=='-')) {						// Still in version area
				if(*pp=='-') { dashed=true; }
				if(dashed && (*pp=='.')) { break; }				// Done versioning
				pp++;
				}
			}
		else if(versioning) {									// Not a digit (probably "mdk", etc.)
			if(dashed) {
				while(isalnum(*pp)) {
					int len=strlen(ver);
					if(len+2<size) {
						ver[len]=*pp; ver[len+1]=0;
						}
					else { break; }									// Out of space
					pp++;
					}
				}
			break;												// Got entire version string
			}
		if(!versioning) {
			pp++;
			}
		}
	}
	
		