// Classes for ezrpm -- By Warren E. Downs
// August 26, 1999

// Purpose: These classes handle directories, packages, capabilities, and files.

#include <stdio.h>
#include <stdlib.h>
#include "ezrpmcls.h"

/*****************************************************************************/	
/* Tfile                                                                     */
/*****************************************************************************/	
Tfile::Tfile(char *name) {
   	this->name=NULL; this->next=NULL;
   	if(name) {
   		this->name=new char[strlen(name)+1];
   		strcpy(this->name,name);	// FIXME: If allocation fails, so will this (and below).
   		}
	}
	
Tfile *Tfile::add(Tfile *file) {
   	if(!file)  { return NULL; }
   	if(!next) {
   		next=file; return file;
   		}
   	else {
   		return next->add(file);
   		}
	}

Tfile::~Tfile() {
   	if(next)  { delete next;	}
   	if(name)  { delete name;	}
   	this->name=NULL; this->next=NULL;
	}

/*****************************************************************************/	
/* Tcapability                                                               */
/*****************************************************************************/	
Tcapability::Tcapability(char *name) {
   	this->name=NULL; this->next=NULL;
   	if(name) {
   		this->name=new char[strlen(name)+1];
   		strcpy(this->name,name);	// FIXME: If allocation fails, so will this (and below).
   		}
	}

Tcapability *Tcapability::add(Tcapability *cap) {
   	if(!cap)  { return NULL; }
   	if(!next) {
   		next=cap; return cap;
   		}
   	else {
   		return next->add(cap);
   		}
	}

Tcapability::~Tcapability() {
   	if(next)  { delete next;	}
   	if(name)  { delete name;	}
   	this->name=NULL; this->next=NULL;
	}

/*****************************************************************************/	
/* Tpackage                                                                  */
/*****************************************************************************/	
Tpackage::Tpackage(Tdir *path, char *name) {
   	this->name=NULL; this->next=NULL; this->caps=NULL; this->files=NULL;
   	this->path=path;
   	if(name) {
   		this->name=new char[strlen(name)+1];
   		strcpy(this->name,name);	// FIXME: If allocation fails, so will this (and below).
   		}
	}
	
Tpackage *Tpackage::add(Tpackage *pkg) {
   	if(!pkg)  { return NULL; }
   	if(!next) {
   		next=pkg; return pkg;
   		}
   	else {
   		return next->add(pkg);
   		}
   	}

Tcapability *Tpackage::add(Tcapability *cap) {
   	if(!cap)  { return NULL; }
   	if(!caps) {
   		caps=cap; return cap;
   		}
   	else {
   		return caps->add(cap);
   		}
	}
	
Tfile *Tpackage::add(Tfile *file) {
   	if(!file)  { return NULL; }
   	if(!files) {
   		files=file; return file;
   		}
   	else {
   		return files->add(file);
   		}
	}
	
Tpackage::~Tpackage() {
   	if(files) { delete files;	}
   	if(caps)  { delete caps;	}
   	if(next)  { delete next;	}
   	if(name)  { delete name;	}
   	this->name=NULL; this->next=NULL; this->caps=NULL; this->files=NULL;
	}
	
/*****************************************************************************/	
/* Tdir                                                                      */
/*****************************************************************************/	
Tdir::Tdir(char *name, char *prompt) {
   	this->name=NULL; this->prompt=NULL; this->next=NULL; this->pkgs=NULL;
   	if(name) {
   		this->name=new char[strlen(name)+1];
   		strcpy(this->name,name);	// FIXME: If allocation fails, so will this (and below).
   		}
   	if(prompt) {
   		this->prompt=new char[strlen(prompt)+1];
   		strcpy(this->prompt,prompt);
   		}
   	}
   	
Tdir *Tdir::add(Tdir *next) {
   	if(!next) { return NULL; }
   	if(!this->next) {
   		this->next=next; return next;
   		}
   	else {
   		return this->next->add(next);
   		}
   	}
   	
Tpackage *Tdir::add(Tpackage *pkg) {
   	if(!pkg) { return NULL; }
   	if(!pkgs) {
   		pkgs=pkg; return pkg;
   		}
   	else {
   		return pkgs->add(pkg);
   		}
   	}
   	
Tdir::~Tdir() {
   	if(next)   { delete next;   }
   	if(pkgs)   { delete pkgs;   }
   	if(prompt) { delete prompt; }
   	if(name)   { delete name;   }
   	this->name=NULL; this->prompt=NULL; this->next=NULL; this->pkgs=NULL;
   	}
	
/*****************************************************************************/	
/* Trpm                                                                      */
/*****************************************************************************/	
	
Trpm::Trpm(char *name) {
	this->name=NULL; this->next=NULL;
	if(name) {
		this->name=new char[strlen(name)+1];
		strcpy(this->name,name);	// FIXME: If allocation fails, so will this (and below).
		}
	}
   	
Trpm *Trpm::add(Trpm *next) {
   	if(!next) { return NULL; }
   	if(!this->next) {
   		this->next=next; return next;
   		}
   	else {
   		return this->next->add(next);
   		}
   	}
   	
Trpm::~Trpm() {
   	if(name) { delete name; name=NULL; }
   	}	
